local addonInfo, Synergy = ...
local syn = Synergy
local nkCreateFrame = EnKai.uiCreateFrame
local debugger = false
local lastbroadcasted = nil
local lastrefreshed = nil
local listed = false
local updated = false
local addonLoaded = false
local activePlayers = {}
local requireUpdate = false

syn.EU = nil
syn.UI = {}
syn.UI.context = UI.CreateContext("mainFrame")
syn.AddonData = Inspect.Addon.Detail("Synergy")
syn.UI.context:SetSecureMode("restricted")
--syn.UI.context:SetSecureMode("restricted")

local icon = UI.CreateContext("syn.UI.mapIcon")
local DataGrid = Yague.DataGrid
local timeInit = os.time()

--[[ 
GUI Contents:

	1. Frame Elements
		- 1. Main Window
			- 1.1 - Side Panel / Options Panel
		- 2. Console Window
			- 2.1 - Console Output Textfield
		- 3. Minimap Icon
	2. Texture Elements
		- 1. Tank Icon
		- 2. Healer Icon
		- 3. Damage Icon
		- 4. Support Icon
	3. Button Elements
		- 1. Join List Button
		- 2. Leave List Button
		- 3. Refresh Button
	4. CheckBox Elements
		- 1. Tank Icon
		- 2. Healer Icon
		- 3. Damage Icon
		- 4. Support Icon
	5. Grid Elements
		- 1. Player Grid
	6. Text Elements
		- 1. Client Name Label
		- 2. Client Prestige Label
		- 3. Client Note Label
		- 4. Status Label
			- 4.1 - Status Value Label
	7. Form Elements
		- Client Note TextField
	8. Events

--]]

-- 1 - Frame Elements
syn.UI.mainWindow = nkCreateFrame("nkWindowModern", "Syn.UI.syn.UI.mainWindow", syn.UI.context)
syn.UI.consoleWindow = nkCreateFrame("nkWindowModern", "Syn.UI.Console", syn.UI.context)
syn.UI.popUpWindow = nkCreateFrame("nkWindowModern", "Syn.UI.PopUpWindow", syn.UI.context)
syn.UI.helpWindow = nkCreateFrame("nkWindowModern", "Syn.UI.HelpWindow", syn.UI.context)
syn.UI.helpPanel = UI.CreateFrame("Frame", "Syn.UI.helpPanel", syn.UI.helpWindow)
syn.UI.optionsPanel = UI.CreateFrame("Frame", "Syn.syn.UI.optionsPanel", syn.UI.mainWindow)
syn.UI.roleError = UI.CreateFrame("Frame", "Syn.syn.UI.optionsPanel", syn.UI.optionsPanel)
syn.UI.toolsPanel = UI.CreateFrame("Frame", "Syn.syn.UI.toolsPanel", syn.UI.mainWindow)

syn.UI.mapIcon = UI.CreateFrame("Texture", "Icon", icon)
syn.UI.mapIconCounter = UI.CreateFrame("Texture", "Icon", icon)

	-- FRAME ELEMENT SETTINGS

		-- 1 - Main Window
		syn.UI.mainWindow:SetTitle("Synergy")
		syn.UI.mainWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0)
		syn.UI.mainWindow:SetAlpha(1)
		syn.UI.mainWindow:SetCloseable(true)
		syn.UI.mainWindow:SetDragable(true)
		syn.UI.mainWindow:SetHeight(600)
		syn.UI.mainWindow:SetWidth(1200)
		syn.UI.mainWindow:SetVisible(false)
		
			-- 1.1 - Side Panel / Options Panel
			syn.UI.optionsPanel:SetPoint("CENTER", syn.UI.mainWindow, "CENTER", 460, 5)
			syn.UI.optionsPanel:SetLayer(4)
			syn.UI.optionsPanel:SetHeight(545)
			syn.UI.optionsPanel:SetWidth(250)
			syn.UI.optionsPanel:SetBackgroundColor(0,0,0,0.8)
			
			-- 1.2 - Tools Panel (Invite Player, Message etc)
			syn.UI.toolsPanel:SetPoint("CENTER", syn.UI.mainWindow, "CENTER", -135, 252)
			syn.UI.toolsPanel:SetLayer(2)
			syn.UI.toolsPanel:SetHeight(50)
			syn.UI.toolsPanel:SetWidth(908)
			syn.UI.toolsPanel:SetBackgroundColor(0,0,0,0.8)
			
			-- 1.3 - Role Error (displayed when a role hasnt be chosen)
			syn.UI.roleError:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, -215)
			syn.UI.roleError:SetLayer(2)
			syn.UI.roleError:SetHeight(90)
			syn.UI.roleError:SetWidth(230)
			syn.UI.roleError:SetBackgroundColor(1,0,0,0.1)
			syn.UI.roleError:SetVisible(false)
			
		-- 2 - Console Window
		syn.UI.consoleWindow:SetTitle("Synergy - Console")
		syn.UI.consoleWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0)
		syn.UI.consoleWindow:SetAlpha(1)
		syn.UI.consoleWindow:SetCloseable(true)
		syn.UI.consoleWindow:SetDragable(true)
		syn.UI.consoleWindow:SetHeight(190)
		syn.UI.consoleWindow:SetWidth(500)
		if not debugger then syn.UI.consoleWindow:SetVisible(false) end

			-- 2.1 - Console Output TextField
			syn.UI.consoleOutput = UI.CreateFrame("RiftTextfield", "Syn.Console.OutputField", syn.UI.consoleWindow)
			syn.UI.consoleOutput:SetPoint("CENTER", syn.UI.consoleWindow, "CENTER", 0, 8)
			syn.UI.consoleOutput:SetWidth(480)
			syn.UI.consoleOutput:SetHeight(140)
			syn.UI.consoleOutput:SetLayer(6)
			syn.UI.consoleOutput:SetBackgroundColor((87/255),(87/255),(87/255),0.8)

		
		-- 3 - Help Window
		syn.UI.helpWindow:SetTitle("Synergy - Help")
		syn.UI.helpWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 450, 450)
		syn.UI.helpWindow:SetAlpha(1)
		syn.UI.helpWindow:SetCloseable(true)
		syn.UI.helpWindow:SetDragable(true)
		syn.UI.helpWindow:SetHeight(400)
		syn.UI.helpWindow:SetWidth(500)
		syn.UI.helpWindow:SetVisible(false)
		
			-- 3.1 - Help Panel
			syn.UI.helpPanel:SetPoint("TOPLEFT", syn.UI.helpWindow, "TOPLEFT", 0, 20)
			syn.UI.helpPanel:SetLayer(3)
			syn.UI.helpPanel:SetHeight(380)
			syn.UI.helpPanel:SetWidth(500)
			syn.UI.helpPanel:SetBackgroundColor(0,0,0,0.8)
		
		-- 4 - Minimap Icon
		syn.UI.mapIcon:SetTexture("Synergy", "img/icon.png")
		syn.UI.mapIcon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, 0)
		syn.UI.mapIcon:SetLayer(3)
		syn.UI.mapIcon:SetWidth(42)
		syn.UI.mapIcon:SetHeight(32)	
		
			-- 4.1 - Minimap Active Players Counter
			syn.UI.mapIconCounter:SetTexture("Synergy", "img/minimap_count.png")
			syn.UI.mapIconCounter:SetPoint("TOPLEFT", syn.UI.mapIcon, "TOPLEFT", -28, 7)
			syn.UI.mapIconCounter:SetLayer(3)
			
		-- 5 - PopUpWindow
		syn.UI.popUpWindow:SetTitle("")
		syn.UI.popUpWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 450, 450)
		syn.UI.popUpWindow:SetAlpha(1)
		syn.UI.popUpWindow:SetCloseable(true)
		syn.UI.popUpWindow:SetDragable(true)
		syn.UI.popUpWindow:SetHeight(190)
		syn.UI.popUpWindow:SetWidth(500)
		syn.UI.popUpWindow:SetVisible(false)
		

-- 2 - Texture Elements
local tankIcon = UI.CreateFrame("Texture", "Tank Icon", syn.UI.mainWindow)
local healerIcon = UI.CreateFrame("Texture", "Healer Icon", syn.UI.mainWindow)
local damageIcon = UI.CreateFrame("Texture", "Damage Icon", syn.UI.mainWindow)
local supportIcon = UI.CreateFrame("Texture", "Support Icon", syn.UI.mainWindow)
local synergyImage = UI.CreateFrame("Texture", "Synergy Help Window Image", syn.UI.helpWindow)
--local errortag = UI.CreateFrame("Texture", "Error Tag", syn.UI.mainWindow)

	-- TEXTURE ELEMENT SETTINGS
	
		-- 1 - Tank Icon
		tankIcon:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", -80, -230)
		tankIcon:SetLayer(6)
		tankIcon:SetHeight(42)
		tankIcon:SetWidth(43)
		tankIcon:SetTextureAsync("Synergy", "img/tank_icon.png")

		-- 2 - Healer Icon
		healerIcon:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", -27, -230)
		healerIcon:SetLayer(6)
		healerIcon:SetHeight(42)
		healerIcon:SetWidth(43)
		healerIcon:SetTextureAsync("Synergy", "img/healer_icon.png")

		-- 3 - Damage Icon
		damageIcon:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 26, -230)
		damageIcon:SetLayer(6)
		damageIcon:SetHeight(42)
		damageIcon:SetWidth(43)
		damageIcon:SetTextureAsync("Synergy", "img/damage_icon.png")

		-- 4 - Support Icon
		supportIcon:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 79, -230)
		supportIcon:SetLayer(6)
		supportIcon:SetHeight(42)
		supportIcon:SetWidth(43)
		supportIcon:SetTextureAsync("Synergy", "img/support_icon.png")
		

		-- 4 - Support Icon
		synergyImage:SetPoint("TOPCENTER", syn.UI.helpPanel, "TOPCENTER", 0, 10)
		synergyImage:SetLayer(9)
		synergyImage:SetWidth(480)
		synergyImage:SetTextureAsync("Synergy", "img/synergy_image.png")
		
		-- 5 - Error Tag
		--errortag:SetPoint("TOPCENTER", syn.UI.optionsPanel, "TOPCENTER", 165, 40)
		--errortag:SetLayer(9)
		--errortag:SetTexture("Synergy", "img/tag.png")
		
-- 3 - Button Elements
local joinListButton = nkCreateFrame("nkButton", "Syn.UI.Button.JoinList", syn.UI.optionsPanel)
local leaveListButton = nkCreateFrame("nkButton", "Syn.UI.Button.LeaveList", syn.UI.optionsPanel)
local refreshButton = nkCreateFrame("nkButton", "Syn.UI.Button.Refresh", syn.UI.optionsPanel)

local invitePlayer = nkCreateFrame("nkButton", "Syn.UI.Button.InvitePlayer", syn.UI.optionsPanel)
local messagePlayer = nkCreateFrame("nkButton", "Syn.UI.Button.MessagePlayer", syn.UI.optionsPanel)

local helpButton = nkCreateFrame("nkButton", "Syn.UI.Button.Help", syn.UI.optionsPanel)

local popUpWindowOK = nkCreateFrame("nkButton", "Syn.UI.Button.PopUpWindowOK", syn.UI.popUpWindow)
local popUpWindowCANCEL = nkCreateFrame("nkButton", "Syn.UI.Button.PopUpWindowCANCEL", syn.UI.popUpWindow)

	-- BUTTON ELEMENT SETTINGS

		-- 1 - Join List Button
		joinListButton:SetText("Join List")
		joinListButton:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, 100)
		joinListButton:SetLayer(5)
		
		-- 2 - Leave List Button
		leaveListButton:SetText("Leave List")
		leaveListButton:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, 145)
		leaveListButton:SetLayer(5)
		
		-- 3 - Refresh Button
		refreshButton:SetText("Refresh")
		refreshButton:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, 190)
		refreshButton:SetLayer(5)
		
		-- 4 - Invite Player Button
		invitePlayer:SetText("Invite Player")
		invitePlayer:SetPoint("RIGHTCENTER", syn.UI.toolsPanel, "RIGHTCENTER", -130, 0)
		invitePlayer:SetLayer(9)
		invitePlayer:SetScale(0.9)
		
		-- 5 - Message Player Button
		messagePlayer:SetText("Message Player")
		messagePlayer:SetPoint("RIGHTCENTER", syn.UI.toolsPanel, "RIGHTCENTER", -10, 0)
		messagePlayer:SetLayer(9)
		messagePlayer:SetScale(0.9)
		
		-- 6 - Addon Help Button
		helpButton:SetText("Addon Help")
		helpButton:SetPoint("RIGHTCENTER", syn.UI.toolsPanel, "RIGHTCENTER", -250, 0)
		helpButton:SetLayer(9)
		helpButton:SetScale(0.9)
		
		-- 6 - Addon Help Button
		popUpWindowOK:SetText("OK")
		popUpWindowOK:SetPoint("CENTER", syn.UI.popUpWindow, "CENTER", -70, 40)
		popUpWindowOK:SetLayer(9)
		
		-- 6 - Addon Help Button
		popUpWindowCANCEL:SetText("CANCEL")
		popUpWindowCANCEL:SetPoint("CENTER", syn.UI.popUpWindow, "CENTER", 70, 40)
		popUpWindowCANCEL:SetLayer(9)
		
-- 4 - CheckBox Elements
local tankCheck = UI.CreateFrame("RiftCheckbox", "tankCheck", syn.UI.mainWindow)
local healerCheck = UI.CreateFrame("RiftCheckbox", "healerCheck", syn.UI.mainWindow)
local damageCheck = UI.CreateFrame("RiftCheckbox", "damageCheck", syn.UI.mainWindow)
local supportCheck = UI.CreateFrame("RiftCheckbox", "supportCheck", syn.UI.mainWindow)

		-- CHECKBOX ELEMENT SETTINGS
			
			-- 1 - Tank CheckBox
			tankCheck:SetLayer(6)
			tankCheck:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", -77, -195)
			
			-- 2 - Healer CheckBox
			healerCheck:SetLayer(6)
			healerCheck:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", -24, -195)
			
			-- 3 - Damage Checkbox
			damageCheck:SetLayer(6)
			damageCheck:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 29, -195)
			
			-- 4 - Support Checkbox
			supportCheck:SetLayer(6)
			supportCheck:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 82, -195)
			
			
-- 5 - Grid Elements
local playerGrid = DataGrid("Syn.UI.PlayerGrid", syn.UI.mainWindow)

	-- GRID ELEMENT SETTINGS

		-- 1 - Player Grid
		playerGrid:SetPadding(1, 1, 1, 1)
		playerGrid:SetHeadersVisible(true)
		playerGrid:SetRowHeight(62)
		playerGrid:SetRowMargin(2)
		playerGrid:SetUnselectedRowBackgroundColor({0.2, 0.2, 0.2, 0.25})
		playerGrid:SetSelectedRowBackgroundColor({0.6, 0.6, 0.6, 0.25})
		playerGrid:SetLayer(5)
		playerGrid:SetHeight(490)
		playerGrid:SetWidth(908)
		playerGrid:AddColumn("clientFullName", "Player", "Text", 100, 1, "clientFullName", true, { Alignment = "center" })
		playerGrid:AddColumn("clientLevel", "Level", "Text", 50, 1, "clientLevel", true, { Alignment = "center" })
		playerGrid:AddColumn("calling", "Calling", "Text", 70, 1, "clientCalling", true, { Alignment = "center" })
		playerGrid:AddColumn("clientVengeance", "Vengeance", "Text", 70, 1, "clientVengeance", true, { Alignment = "center" })
		playerGrid:AddColumn("clientValor", "Valor", "Valor", 50, 1, "clientValor", true, { Alignment = "center" })
		playerGrid:AddColumn("clientRoles", "Role", "Text", 50, 1, "clientRoles", true, { Alignment = "center" })
		playerGrid:AddColumn("clientPrestige", "Prestige", "Text", 50, 1, "clientPrestige", true, { Alignment = "center" })
		playerGrid:AddColumn("clientNote", "Note", "Text", 150, 1, "clientNote", true, { Alignment = "left" })
		playerGrid:GetInternalContent():SetBackgroundColor(0, 0.05, 0.05, 0.5)
		playerGrid:SetLoadingBarEnabled(true)
		playerGrid:SetPoint("CENTER", syn.UI.mainWindow, "CENTER", -135, -23)
		
-- 6 - Text Elements
local miniMapCounterText = UI.CreateFrame("Text", "Minimap Active Player Count", syn.UI.mapIconCounter)
local clientName = UI.CreateFrame("Text", "Clients Name", syn.UI.mainWindow)
local clientPrestige = UI.CreateFrame("Text", "Clients Prestige", syn.UI.mainWindow)
local clientNoteLabel = UI.CreateFrame("Text", "Clients Note Label", syn.UI.mainWindow)
local clientNoteLabel2 = UI.CreateFrame("Text", "Clients Note Label2", syn.UI.mainWindow)
local statusText = UI.CreateFrame("Text", "Listing Status Label", syn.UI.mainWindow)
local statusValue = UI.CreateFrame("Text", "Listing Status Value", syn.UI.mainWindow)

local activePlayersLabel = UI.CreateFrame("Text", "Active Players Label", syn.UI.mainWindow)
local activePlayersValue = UI.CreateFrame("Text", "Active Players Value", syn.UI.mainWindow)

local helpText1 = UI.CreateFrame("Text", "Version Info", syn.UI.helpWindow)
local helpText2 = UI.CreateFrame("Text", "Help Paragraph1", syn.UI.helpWindow)
local helpText3 = UI.CreateFrame("Text", "Help Paragraph2", syn.UI.helpWindow)

local errortagLabel = UI.CreateFrame("Text", "Error Tag Label", syn.UI.optionsPanel)

local popUpText = UI.CreateFrame("Text", "Pop up Label", syn.UI.popUpWindow)

	-- TEXT ELEMENT SETTINGS
	
		-- 1 - Client Name
		clientName:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, -130)
		clientName:SetLayer(6)
		clientName:SetFontSize(16)
		
		-- 2 - Client Prestige
		clientPrestige:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, -90)
		clientPrestige:SetLayer(6)
		clientPrestige:SetFontSize(28)
		
		-- 3 - Client Note Label
		clientNoteLabel:SetText("Set your personal note:")
		clientNoteLabel:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, -10)
		clientNoteLabel:SetLayer(6)
		clientNoteLabel2:SetText("This will be broadcasted to other users")
		clientNoteLabel2:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, 50)
		clientNoteLabel2:SetLayer(6)
		
		-- 4 - Status Label
		statusText:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", -50, 240)
		statusText:SetLayer(6)
		statusText:SetFontSize(14)
		statusText:SetText("Status:")
			
			-- 4.1 - Status Text Value
			statusValue:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 40, 240)
			statusValue:SetLayer(6)
			statusValue:SetFontSize(14)
			statusValue:SetText("UNLISTED")
			statusValue:SetFontColor(1,0,0)
		
		-- 5 - Client Note Label
		activePlayersLabel:SetText("Total Active Players:")
		activePlayersLabel:SetPoint("LEFTCENTER", syn.UI.toolsPanel, "LEFTCENTER", 10, 0)
		activePlayersLabel:SetLayer(6)
		activePlayersValue:SetPoint("LEFTCENTER", syn.UI.toolsPanel, "LEFTCENTER", 125, 0)
		activePlayersValue:SetLayer(6)
		
		-- 6 - Help Text

			-- 6.1 - Help Text 1
			helpText1:SetText("Version: "..addonInfo.toc.Version.." - Written by Zanderwar@Wolfsbane")
			helpText1:SetPoint("CENTER", syn.UI.helpWindow, "CENTER", 0, -30)
			helpText1:SetLayer(6)
			helpText1:SetFontSize(14)
			
			-- 6.1 - Help Text 2
			helpText2:SetText("To begin using the addon, simply select a role from the main window,\nenter a personal note if you would like other players to know something and\nthen click the 'Join List' button. If you would like to leave the list then simply\nclick the relevant button (changes take up to 60 seconds to occur)\n\nYou will be automatically be removed from the list if you disconnect.\n\nSPREAD THE WORD! - The more people who use this addon the better the\nPVP community will become!")
			helpText2:SetPoint("LEFTCENTER", syn.UI.helpWindow, "LEFTCENTER", 10, 90)
			helpText2:SetLayer(6)
			helpText2:SetFontSize(14)
			
		-- 7 Minimap Icon Counter
		miniMapCounterText:SetText("0")
		miniMapCounterText:SetPoint("CENTER", syn.UI.mapIconCounter, "CENTER", -4, 1)
		miniMapCounterText:SetLayer(6)
		miniMapCounterText:SetFontSize(10)
		
		-- 8 - Client Note Label
		errortagLabel:SetText("YOU HAVN'T SELECTED A ROLE:")
		errortagLabel:SetPoint("TOPCENTER", syn.UI.optionsPanel, "TOPCENTER", 0, 0)
		errortagLabel:SetLayer(20)
		errortagLabel:SetVisible(false)
		
		-- 9 - Popup Text
		popUpText:SetText("Are you sure you want to be a jackass?")
		popUpText:SetPoint("TOPCENTER", syn.UI.popUpWindow, "TOPCENTER", 0, 50)
		popUpText:SetLayer(20)
		popUpText:SetFontSize(14)
		
-- 7 - Form Elements
local clientNote = UI.CreateFrame("RiftTextfield", "Clients Broadcast Note", syn.UI.mainWindow)

	-- FORM ELEMENT SETTINGS
	clientNote:SetPoint("CENTER", syn.UI.optionsPanel, "CENTER", 0, 20)
	clientNote:SetWidth(230)
	clientNote:SetLayer(6)
	clientNote:SetBackgroundColor((87/255),(87/255),(87/255),0.8)


-- 8 - Events
Command.Event.Attach(EnKai.events['Syn.UI.Button.LeaveList'].Clicked, function () 
	if not syn.channelCheck() then return end
	if listed then
		listed = false
		statusValue:SetText("UNLISTED")
		statusValue:SetFontColor(1,0,0)
	end

end,"Syn.UI.Button.LeaveList.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.JoinList'].Clicked, function () 
	if not syn.channelCheck() then return end
	if not listed then
	
		if tankCheck:GetChecked()
		or healerCheck:GetChecked()
		or damageCheck:GetChecked()
		or supportCheck:GetChecked() then
			listed = true
			statusValue:SetText("LISTED")
			statusValue:SetFontColor(0,1,0)
			
			errortagLabel:SetVisible(false)
			syn.UI.roleError:SetVisible(false)
		else
			print("You must select a role(s) you're willing to play.")
			errortagLabel:SetVisible(true)
			syn.UI.roleError:SetVisible(true)
		end
	end

end,"Syn.UI.Button.JoinList.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.Refresh'].Clicked, function () 
	if not syn.channelCheck() then return end
	local now = os.time()
	
	if lastrefreshed == nil or (now - lastrefreshed) > 60 then
		lastrefreshed = now
		syn.broadcast("refresh")
	else
		print("Error: You may only refresh once every 60 seconds. It has only been "..(now-lastrefreshed).." seconds")
	end

end,"Syn.UI.Button.JoinList.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.Help'].Clicked, function () 
	
	if syn.UI.helpWindow:GetVisible() == false then
		syn.UI.helpWindow:SetVisible(true)
	else
		syn.UI.helpWindow:SetVisible(false)
	end
end,"Syn.UI.Button.Help.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.InvitePlayer'].Clicked, function () 
	
	local key, data = playerGrid:GetSelectedData()
	local macro = ""
	if data == nil then return end
	if data["clientName"] == Inspect.Unit.Detail("player").name then
		print("Error: You cannot invite yourself.")
	else
		if Inspect.Shard().name == data["clientShard"] then
			macro = ("invite ".. data["clientName"])
			syn.UI.popUpWindow:SetTitle("Inviting "..data["clientName"])
		else
			macro = ("invite ".. data["clientFullName"])
			syn.UI.popUpWindow:SetTitle("Inviting "..data["clientFullName"])
		end
		syn.UI.popUpWindow:SetSecureMode("restricted")
		popUpWindowOK:SetSecureMode("restricted")
		syn.debug("Macro: "..macro.." set to OK button")
		popUpWindowOK:EventMacroSet(Event.UI.Input.Mouse.Left.Click, macro)
		
		popUpText:SetText("Are you sure you want to invite this player?")
		syn.UI.popUpWindow:SetVisible(true)
	end
	
end,"Syn.UI.Button.InvitePlayer.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.MessagePlayer'].Clicked, function () 
	
	local key, data = playerGrid:GetSelectedData()
	local macro = ""
	if data == nil then return end
	if data["clientName"] == Inspect.Unit.Detail("player").name then
		print("Error: You cannot message yourself.")
	else
		if Inspect.Shard().name == data["clientShard"] then
			macro = ("tell ".. data["clientName"] .." [Synergy] I am requesting to group up with you!")
			syn.UI.popUpWindow:SetTitle("Messaging "..data["clientName"])
		else
			macro = ("tell ".. data["clientFullName"] .." [Synergy] I am requesting to group up with you!")
			syn.UI.popUpWindow:SetTitle("Messaging "..data["clientFullName"])
		end
		syn.UI.popUpWindow:SetSecureMode("restricted")
		popUpWindowOK:SetSecureMode("restricted")
		syn.debug("Macro: "..macro.." set to OK button")
		popUpWindowOK:EventMacroSet(Event.UI.Input.Mouse.Left.Click, macro)
		
		popUpText:SetText("Are you sure you want to message this player?")
		syn.UI.popUpWindow:SetVisible(true)
	end
	
end,"Syn.UI.Button.Help.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.PopUpWindowOK'].Clicked, function () 
	
	syn.UI.popUpWindow:SetVisible(false)
	
end,"Syn.UI.Button.PopUpWindowOK.Clicked")

Command.Event.Attach(EnKai.events['Syn.UI.Button.PopUpWindowCANCEL'].Clicked, function () 
	
	syn.UI.popUpWindow:SetVisible(false)
	
end,"Syn.UI.Button.PopUpWindowOK.Clicked")
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------- END OF GUI --------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------


--local values = {}
--values[1] = { player = "Zanderwar@Wolfsbane", level = "60", role = "THDS", prestige = "33", note = "Great player, full myrm" }
--values[2] = { player = "col1", level = "col2", role = "col3", prestige = "col4", note = "col5" }
--values[3] = { player = "col1", level = "col2", role = "col3", prestige = "col4", note = "col5" }
--playerGrid:SetData(values, nil)


local function AttachDragFrame(parent, name, layer, button)

	if not name then name = "" end
	if not layer then layer = 0 end
	if not button then button = "LeftDown" end
	
	local DragFrame = UI.CreateFrame("Frame", name.."_DragFrame", parent)
	DragFrame:SetPoint("TOPLEFT", parent, "TOPLEFT")
	DragFrame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT")
	DragFrame.parent = parent
	DragFrame.MouseDown = false
	DragFrame:SetLayer(layer)
	
	if button == "LeftDown" then
		function DragFrame.Event:LeftDown()
			self.MouseDown = true
			mouseData = Inspect.Mouse()
			self.MyStartX = self.parent:GetLeft()
			self.MyStartY = self.parent:GetTop()
			self.StartX = mouseData.x - self.MyStartX
			self.StartY = mouseData.y - self.MyStartY
			tempX = self.parent:GetLeft()
			tempY = self.parent:GetTop()
			tempW = self.parent:GetWidth()
			tempH =	self.parent:GetHeight()
			self.parent:ClearAll()
			self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tempX, tempY)
			self.parent:SetWidth(tempW)
			self.parent:SetHeight(tempH)
		end
		function DragFrame.Event:MouseMove(mouseX, mouseY)
			if self.MouseDown then
				self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.StartX), (mouseY - self.StartY))
			end
		end
		function DragFrame.Event:LeftUp()
			if self.MouseDown then
				self.MouseDown = false
			end
		end
	elseif button == "RightDown" then
		function DragFrame.Event:RightDown()
			self.MouseDown = true
			mouseData = Inspect.Mouse()
			self.MyStartX = self.parent:GetLeft()
			self.MyStartY = self.parent:GetTop()
			self.StartX = mouseData.x - self.MyStartX
			self.StartY = mouseData.y - self.MyStartY
			tempX = self.parent:GetLeft()
			tempY = self.parent:GetTop()
			tempW = self.parent:GetWidth()
			tempH =	self.parent:GetHeight()
			self.parent:ClearAll()
			self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tempX, tempY)
			self.parent:SetWidth(tempW)
			self.parent:SetHeight(tempH)
		end
		function DragFrame.Event:MouseMove(mouseX, mouseY)
			if self.MouseDown then
				self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", (mouseX - self.StartX), (mouseY - self.StartY))
			end
		end
		function DragFrame.Event:RightUp()
			if self.MouseDown then
				self.MouseDown = false
			end
		end

	end
	
	return DragFrame
end

syn.UI.MapIconDragFrame = AttachDragFrame(syn.UI.mapIcon, "WFWMapIcon", 10, "RightDown")
syn.UI.MapIconDragFrame:SetVisible(true)

function syn.ignoreme()
end

function syn.initAddon()
	if not addonLoaded then
		addonLoaded = true
		
		Command.System.Watchdog.Quiet()
		Command.Console.Display("general", false, "<font color=\"#0000FF\">Loaded.</font>", true)
		
		Command.Message.Accept("channel", "Synergy")
		Command.Message.Accept("tell", "Synergy")
		
		syn.UI.consoleOutput:SetText("Addon Loaded")
		
		syn.channelCheck()
	end

end

function syn.channelCheck()
	local shard = string.lower(Inspect.Shard().name)
	
	if (shard == 'faeblight' or shard == 'laethys' or shard == 'hailol' or shard == 'wolfsbane' or shard == 'deepwood' or shard == 'greybriar' or shard == 'seastone') then syn.EU = false end
	if (shard == 'bloodiron' or shard == 'brutwacht' or shard == 'trübkopf' or shard == 'brutmutter' or shard == 'brisesol' or shard == 'phynnious' or shard == 'gelidra' or shard == 'zaviel') then syn.EU = true end
	return true
	
end

function syn.updateClientInfo(unit)

	for k,v in pairs(unit) do if v ~= "player" then return end end -- Check if the player data is fully loaded
		
	local unit = Inspect.Unit.Detail("player")
	local prestige = Inspect.Pvp.Prestige()
	
	if unit and prestige then
		local playerName = unit.name
		playerLevel = unit.level
		if playerLevel == nil then playerLevel = "N/A" end
		playerTitleLabel = playerName.." ("..playerLevel..")"
		clientName:SetText(playerTitleLabel)
		clientPrestige:SetText("Prestige "..prestige.rank)
	end

end


function syn.broadcast(params) 
	local channel = ""
	
	local stat = Inspect.Stat()
	
	if not stat then return end
	
	if syn.EU then channel = "CrossEvents@Brutwacht" else channel = "CrossEvents@Faeblight" end
	
	local data = {}
	
	if params == nil then
		
		local roles = ""
		data["clientName"] = Inspect.Unit.Detail("player").name
		data["clientShard"] = Inspect.Shard().name
		data["clientFullName"] = (data["clientName"] .. "@" .. data["clientShard"])
		data["clientCalling"] = firstToUpper(Inspect.Unit.Detail("player").calling) or nil
		
		if tankCheck:GetChecked() then roles = "T" end
		if healerCheck:GetChecked() then roles = roles .. "H" end
		if damageCheck:GetChecked() then roles = roles .."D" end
		if supportCheck:GetChecked() then roles = roles .. "S" end
		
		if roles == "" then roles = "N/A" end
		
		data["clientRoles"] = roles
		data["clientVengeance"] = Inspect.Stat().vengeance or nil
		data["clientValor"] = Inspect.Stat().valor or nil
		data["clientLevel"] = Inspect.Unit.Detail("player").level or nil
		data["clientPrestige"] = Inspect.Pvp.Prestige().rank
		data["clientNote"] = clientNote:GetText()
		data["lastBroadcast"] = os.time()
		
		
	elseif params == "remove" then
		data["request"] = "removal"
		data["clientName"] = Inspect.Unit.Detail("player").name
	elseif params == "refresh" then
		data["request"] = "refresh"
	end
	
	local serialized = Utility.Serialize.Inline(data)
	data = zlib.deflate()(serialized, "finish")
	
	if (Inspect.Shard().name == "Faeblight") then channel = "CrossEvents" end
	
	Command.Message.Broadcast("channel", channel, "Synergy", data)
		
	
end

function syn.debug(data)
	if debugger ~= true then return end
	local dump = syn.UI.consoleOutput:GetText()
	
	--table.foreach(data, print)
	
	--for k, v in pairs(data) do syn.UI.consoleOutput:SetText(tostring(k).."  "..tostring(v)) end 
	
	if dump == nil then
		syn.UI.consoleOutput:SetText(data)
	else
		syn.UI.consoleOutput:SetText(tostring(data) .."\n".. dump)
	end
end

function syn.receive(from, type, channel, identifier, incoming) 
	if identifier ~= "Synergy" then return end
	if from == Inspect.Unit.Detail("player").name then 
		--if not debugger then return end
	end
	local decompressed = zlib.inflate()(incoming, "finish")
	data = {}		
	local deserialize = loadstring("return " .. decompressed)()
	data = deserialize
	
	if debugger then syn.debug(from .. ": " ..decompressed) end
			
	
	if data['request'] ~= nil and data['request'] == 'removal' then
		-- Remove sender from active list
		local newList = {}
		for k,v in ipairs(activePlayers) do
			if v["clientName"] == data["clientName"] then
				syn.debug("- Removed broadcaster from list")
			else
				newList[k] = v
			end
			
		end
		
		activePlayers = newList
		syn.setGridData()
		  
	end
	
	syn.updatePlayerGrid(data)
	
	if data['request'] ~= nil and data['request'] == 'refresh' then
		if listed then
			lastbroadcasted = os.time()
			syn.broadcast()
		end
	end
		
end

function syn.flushInactivePlayers(forceupdate)
	
	if forceupdate then requireUpdate = true end
	
	local timeNow = os.time()
	local newList = {}

	
	for k,v in ipairs(activePlayers) do
		if (timeNow - v["lastBroadcast"]) > 60 then
			-- Simply wont add this client back to the list.
			requireUpdate = true
			syn.debug("Removed "..v["clientName"].." from the list due to inactivity.")
		else
			table.insert(newList, v)
		end
	end
	
	activePlayers = newList
	
	if requireUpdate then
		syn.setGridData()
		requireUpdate = false
	end
end
function syn.updatePlayerGrid(data)
	syn.debug("Func: syn.updatePlayerGrid()")
	if data and data["clientFullName"] ~= nil then
		-- Add to list if not already exist
		
		for k,v in ipairs(activePlayers) do
			
			if v["clientFullName"] == data["clientFullName"] or v["clientName"] == data["clientName"] then
				-- Client already in list
				data["lastBroadcast"] = os.time()
				activePlayers[k] = data
				requireUpdate = true
				syn.debug("-- Client detected in list already, ignoring")
				return
			end
			
		end
		
		syn.debug("- Added recent broadcaster to list")
		table.insert(activePlayers, data)
		

		syn.flushInactivePlayers(true)
	end
	
	
end

function syn.setGridData() 
	playerGrid:SetData({})
	playerGrid:SetData(activePlayers)
	playerGrid:SetSelectedKey(1)
end
function syn.slashHandler(handle, params)
	
	if params == "help" then
		syn.UI.helpWindow:SetVisible(true)
	elseif params == "show" then
		syn.UI.mainWindow:SetVisible(true)
	else
		print("------------")
		print("Synergy Commands:")
		print("Either /syn <command> or /synergy <command> may be used.")
		print("\t- help - Will display the addon help window.")
		print("\t- show - Will show the main addon screen")
		print("------------")
	end
	
end

function firstToUpper(str)
	if str == nil then return end
    return (str:gsub("^%l", string.upper))
end

function syn.saveVariables(handle, addon)
	if addon == "Synergy" then
		Syn_framePosX = math.floor(syn.UI.mainWindow:GetLeft())
		Syn_framePosY = math.floor(syn.UI.mainWindow:GetTop())
		Syn_iconX = math.floor(syn.UI.mapIcon:GetLeft())
		Syn_iconY = math.floor(syn.UI.mapIcon:GetTop())
	end
end

function syn.initVariables(handle, addon)
	if addon == "Synergy" then
		local Syn_frameY = 350
		local Syn_frameY = 350
		local Syn_mapIconX = 400
		local Syn_mapIconY = 400
				
		if Syn_framePosX then
			Syn_frameX = Syn_framePosX
		end
		
		if Syn_framePosY then
			Syn_frameY = Syn_framePosY
		end
		
		if Syn_iconX then
			Syn_mapIconX = Syn_iconX
		end
		
		if Syn_iconY then
			Syn_mapIconY = Syn_iconY
		end

		
		syn.UI.mainWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Syn_frameX, Syn_frameY)
		syn.UI.mapIcon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Syn_mapIconX, Syn_mapIconY)
		
		
	end
end

syn.UI.MapIconDragFrame:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
	if syn.UI.mainWindow:GetVisible() == true then
		syn.UI.mainWindow:SetVisible(false)
		Command.Console.Display("general", false, "Synergy has been hidden.", true)
	else
		syn.UI.mainWindow:SetVisible(true)
		Command.Console.Display("general", false, "Synergy is now showing.", true)
	end
end, "Event.UI.Input.Mouse.Left.Click")

function syn.update()
	now = os.time()
	if (now - timeInit) > 1 and updated == false then
		-- To ensure rift has loaded all the clients stats
		updated = true
		
		if listed then syn.broadcast() end
		
		syn.broadcast("refresh")
	end
	
	local playerCount = #activePlayers
	if playerCount == nil then 
		playerCount = 0 
		syn.UI.mapIconCounter:SetVisible(false)
		activePlayersValue:SetText("0")
	elseif playerCount > 1 and listed then 
		playerCount1 = playerCount .. " (incl. you)"
		miniMapCounterText:SetText(tostring(playerCount))
		activePlayersValue:SetText(tostring(playerCount1))
		syn.UI.mapIconCounter:SetVisible(true)
	elseif playerCount == 1 and listed then
		playerCount1 = playerCount .. " (incl. you)"
		activePlayersValue:SetText(tostring(playerCount1))
		syn.UI.mapIconCounter:SetVisible(false)
	else
		activePlayersValue:SetText("0")
		syn.UI.mapIconCounter:SetVisible(false)
	end
	

		
	
	if listed then
		if lastbroadcasted == nil or (now - lastbroadcasted) > 30 then
			lastbroadcasted = os.time()
			
			syn.broadcast()
		end
	end
	
	syn.flushInactivePlayers()
	
end
Command.Event.Attach(Event.Addon.Load.End, syn.initAddon, "syn.InitAddon")
Command.Event.Attach(Command.Slash.Register("synergy"), syn.slashHandler, "syn.slashHandler")
Command.Event.Attach(Command.Slash.Register("syn"), syn.slashHandler, "syn.slashHandler")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, syn.saveVariables, "syn.saveVariables")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, syn.initVariables, "syn.initVariables")
table.insert(Event.Message.Receive, 				{syn.receive, 			"Synergy", "syn.receive"})
table.insert(Event.System.Update.Begin, 		{syn.update, 		"Synergy", "syn.update" })
table.insert(Event.Unit.Availability.Full, 		{syn.updateClientInfo, 		"Synergy", "syn.updateClientInfo" })